@@ -1,6 +1,6 @@
 from __future__ import annotations
 
-from koldstart.api import KoldstartHost, LocalHost, isolated
+from koldstart.api import KoldstartHost, LocalHost, cached, isolated
 from koldstart.sdk import CloudKeyCredentials
 
 local = LocalHost()
