@@ -1,6 +1,8 @@
 from __future__ import annotations
 
 from datetime import datetime
+from sys import argv
+from uuid import uuid4
 
 import click
 import koldstart.auth as auth
@@ -9,16 +11,79 @@ from koldstart.console import console
 from koldstart.exceptions import ApplicationExceptionHandler
 from rich.table import Table
 
+from .logging import get_logger, set_debug_logging
+from .logging.trace import get_tracer
+
 DEFAULT_HOST = "api.alpha.fal.ai"
 HOST_ENVVAR = "KOLDSTART_HOST"
 
 DEFAULT_PORT = "443"
 PORT_ENVVAR = "KOLDSTART_PORT"
 
+DEBUG_ENABLED = False
 
-@click.group(cls=ApplicationExceptionHandler)
+
+log = get_logger(__name__)
+
+
+class ExecutionInfo:
+
+    debug: bool
+    invocation_id: str
+
+    def __init__(self, debug=False):
+        self.debug = debug
+        self.invocation_id = str(uuid4())
+
+
+class MainGroup(click.Group):
+    """A custom implementation of the top-level group
+    (i.e. called on all commands and subcommands).
+
+    This implementation allows for centralized behavior, including
+    exception handling.
+    """
+
+    _exception_handler = ApplicationExceptionHandler()
+
+    _tracer = get_tracer(__name__)
+
+    def invoke(self, ctx):
+        execution_info = ExecutionInfo(debug=ctx.params["debug"])
+        qualified_name = " ".join([ctx.info_name] + argv[1:])
+        invocation_id = execution_info.invocation_id
+        set_debug_logging(execution_info.debug)
+
+        with self._tracer.start_as_current_span(
+            qualified_name, attributes={"invocation_id": invocation_id}
+        ):
+            try:
+                log.debug(
+                    f"Executing command",
+                    command=qualified_name,
+                    invocation_id=invocation_id,
+                )
+                return super().invoke(ctx)
+            except Exception as exception:
+                log.error(exception)
+                if execution_info.debug:
+                    # Here we supress detailed errors on click lines because
+                    # they're mostly decorator calls, irrelevant to the dev's error tracing
+                    console.print_exception(suppress=[click])
+                    console.print()
+                    console.print(
+                        f"The [markdown.code]invocation_id[/] for this operation is: [white]{invocation_id}[/]"
+                    )
+                else:
+                    self._exception_handler.handle(exception)
+
+
+@click.group(cls=MainGroup)
+@click.option(
+    "--debug", is_flag=True, help="Enable detailed errors and verbose logging."
+)
 @click.version_option()
-def cli():
+def cli(debug):
     pass
 
 
